#!/usr/bin/env python26

__author__ = 'Aaron Mildenstein'

#
# Deletes all indices with a datestamp older than "daysToKeep" for daily
# if you have hourly indices, it will delete all of those older than "hoursToKeep"
# 
# This script presumes an index is named typically, e.g. logstash-YYYY.MM.DD
# It will work with any name-YYYY.MM.DD or name-YYYY.MM.DD.HH type sequence
#
# Requires python 2.6 or python 2.4 plus simplejson module (according to pyes docs)
# Download: http://pypi.python.org/pypi/pyes/
# Github: https://github.com/aparo/pyes
# Documentation: http://packages.python.org/pyes/
# 

import time
from datetime import timedelta
from pyes import *


# ES setup

esHost = '127.0.0.1'

# pyes is capable of using the thrift protocol, so use the appropriate port in that case
esPort = '9200'

# How long to keep indices?

# Daily
daysToKeep = 31

# Hourly
hoursToKeep = 25

# Since indexes are created on Zulu time (UTC), 
# compensate by adding the timezone differential via time.altzone
now = time.time() + time.altzone

monthlyCutoff = now - (daysToKeep * 24 * 60 * 60)
hourlyCutoff = now - (hoursToKeep * 60 * 60)

def idxEpoch(idxName):
    ymd = idxName.split('-')[1].split('.')
    h = 0
    if len(ymd) == 4: 
        h = int(ymd[3])
    return time.mktime((int(ymd[0]), int(ymd[1]), int(ymd[2]),h,0,0,0,0,0))


conn = ES(esHost + ':' + esPort,timeout=10.0)
indices = conn.get_indices()
indexlist = indices.keys()

print
print "Deleting daily indices older than ", daysToKeep, " days."
print "Deleting hourly indices older than ", hoursToKeep, " hours."
print

for idxName in sorted(set(indexlist)):
    idxTime = idxEpoch(idxName)
    if len(idxName.split('-')[1].split('.')) < 4:
        compare = monthlyCutoff
    else:
        compare = hourlyCutoff
    if idxTime < compare:
        print idxName, timedelta(seconds=(compare - idxEpoch(idxName))), "Older than cutoff"
        print "Deleting index: ",  idxName
        retval = conn.delete_index_if_exists(idxName)
         
# >>> retval will look like this if successful:
# >>> {u'acknowledged': True, u'ok': True}
         
        if retval.get('ok'):
            print "Successfully deleted index: ", idxName
        else:
            print "Error deleting index: ", idxName
    else:
        print idxName, timedelta(seconds=(idxEpoch(idxName) - compare)), "remaining"

